@@ -1,8 +1,8 @@
-import os
+from pathlib import Path
 
 __all__ = ["available", "get_path"]
 
-_module_path = os.path.dirname(__file__)
+_module_path = Path(__file__).resolve().parent
 available = ["bubenec", "tests"]
 
 
@@ -16,7 +16,9 @@ def get_path(dataset):
         all options.
     """
     if dataset in available:
-        return os.path.abspath(os.path.join(_module_path, dataset + ".gpkg"))
-    msg = f"The dataset '{dataset}' is not available. "
-    msg += "Available datasets are {}".format(", ".join(available))
+        return str(_module_path / (dataset + ".gpkg"))
+    msg = (
+        f"The dataset {dataset!r} is not available. "
+        f"Available datasets are {', '.join(available)}"
+    )
     raise ValueError(msg)
