@@ -5,10 +5,12 @@
 
 import math
 
+import geopandas as gpd
 import networkx as nx
 import numpy as np
 import pandas as pd
 import shapely
+from packaging.version import Version
 from tqdm.auto import tqdm  # progress bar
 
 from .utils import _azimuth
@@ -27,6 +29,8 @@ __all__ = [
     "Neighbors",
 ]
 
+GPD_GE_013 = Version(gpd.__version__) >= Version("0.13.0")
+
 
 class Orientation:
     """
@@ -70,7 +74,9 @@ class Orientation:
 
         bboxes = shapely.minimum_rotated_rectangle(gdf.geometry)
         for geom, bbox in tqdm(
-            zip(gdf.geometry, bboxes), total=gdf.shape[0], disable=not verbose
+            zip(gdf.geometry, bboxes, strict=True),
+            total=gdf.shape[0],
+            disable=not verbose,
         ):
             if geom.geom_type in ["Polygon", "MultiPolygon", "LinearRing"]:
                 bbox = list(bbox.exterior.coords)
@@ -132,7 +138,10 @@ class SharedWalls:
     def __init__(self, gdf):
         self.gdf = gdf
 
-        inp, res = gdf.sindex.query_bulk(gdf.geometry, predicate="intersects")
+        if GPD_GE_013:
+            inp, res = gdf.sindex.query(gdf.geometry, predicate="intersects")
+        else:
+            inp, res = gdf.sindex.query_bulk(gdf.geometry, predicate="intersects")
         left = gdf.geometry.take(inp).reset_index(drop=True)
         right = gdf.geometry.take(res).reset_index(drop=True)
         intersections = left.intersection(right).length
@@ -544,7 +553,7 @@ class NeighborDistance:
             if geom is not None and index in spatial_weights.neighbors:
                 neighbours = spatial_weights.neighbors[index]
                 building_neighbours = data.loc[neighbours]
-                if len(building_neighbours) > 0:
+                if len(building_neighbours):
                     results_list.append(
                         building_neighbours.geometry.distance(geom).mean()
                     )
@@ -687,7 +696,10 @@ class NeighboringStreetOrientationDeviation:
         self.gdf = gdf
         self.orientation = gdf.geometry.apply(self._orient)
 
-        inp, res = gdf.sindex.query_bulk(gdf.geometry, predicate="intersects")
+        if GPD_GE_013:
+            inp, res = gdf.sindex.query(gdf.geometry, predicate="intersects")
+        else:
+            inp, res = gdf.sindex.query_bulk(gdf.geometry, predicate="intersects")
         itself = inp == res
         inp = inp[~itself]
         res = res[~itself]
@@ -793,7 +805,9 @@ class BuildingAdjacency:
             print("Spatial weights ready...") if verbose else None
 
         self.sw = spatial_weights
-        patches = dict(zip(gdf[unique_id], spatial_weights.component_labels))
+        patches = dict(
+            zip(gdf[unique_id], spatial_weights.component_labels, strict=True)
+        )
 
         for uid in tqdm(
             self.id,
