@@ -11,6 +11,8 @@ import numpy as np
 import pandas as pd
 import shapely
 from packaging.version import Version
+from scipy.signal import find_peaks
+from scipy.stats import gaussian_kde
 from shapely.geometry import LineString, Point
 from shapely.ops import linemerge, polygonize
 from tqdm.auto import tqdm
@@ -25,10 +27,10 @@ __all__ = [
     "close_gaps",
     "extend_lines",
     "roundabout_simplification",
+    "FaceArtifacts",
 ]
 
-GPD_10 = Version(gpd.__version__) >= Version("0.10")
-GPD_09 = Version(gpd.__version__) >= Version("0.9")
+GPD_GE_013 = Version(gpd.__version__) >= Version("0.13.0")
 
 
 def preprocess(
@@ -77,11 +79,7 @@ def preprocess(
         GeoDataFrame containing preprocessed geometry
     """
     blg = buildings.copy()
-    if GPD_10:
-        blg = blg.explode(ignore_index=True)
-    else:
-        blg = blg.explode()
-        blg.reset_index(drop=True, inplace=True)
+    blg = blg.explode(ignore_index=True)
     for loop in range(0, loops):
         print("Loop", loop + 1, f"out of {loops}.") if verbose else None
         blg.reset_index(inplace=True, drop=True)
@@ -104,13 +102,7 @@ def preprocess(
             if size and row.geometry.area < size:
                 if row.n_count == 1:
                     uid = blg.iloc[row.neighbors[0]].mm_uid
-
-                    if uid in join:
-                        existing = join[uid]
-                        existing.append(row.mm_uid)
-                        join[uid] = existing
-                    else:
-                        join[uid] = [row.mm_uid]
+                    join.setdefault(uid, []).append(row.mm_uid)
                 elif row.n_count > 1:
                     shares = {}
                     for n in row.neighbors:
@@ -119,23 +111,13 @@ def preprocess(
                         ).length
                     maximal = max(shares.items(), key=operator.itemgetter(1))[0]
                     uid = blg.loc[maximal].mm_uid
-                    if uid in join:
-                        existing = join[uid]
-                        existing.append(row.mm_uid)
-                        join[uid] = existing
-                    else:
-                        join[uid] = [row.mm_uid]
+                    join.setdefault(uid, []).append(row.mm_uid)
                 else:
                     delete.append(row.Index)
             if compactness and row.circu < compactness:
                 if row.n_count == 1:
                     uid = blg.iloc[row.neighbors[0]].mm_uid
-                    if uid in join:
-                        existing = join[uid]
-                        existing.append(row.mm_uid)
-                        join[uid] = existing
-                    else:
-                        join[uid] = [row.mm_uid]
+                    join.setdefault(uid, []).append(row.mm_uid)
                 elif row.n_count > 1:
                     shares = {}
                     for n in row.neighbors:
@@ -144,12 +126,7 @@ def preprocess(
                         ).length
                     maximal = max(shares.items(), key=operator.itemgetter(1))[0]
                     uid = blg.loc[maximal].mm_uid
-                    if uid in join:
-                        existing = join[uid]
-                        existing.append(row.mm_uid)
-                        join[uid] = existing
-                    else:
-                        join[uid] = [row.mm_uid]
+                    join.setdefault(uid, []).append(row.mm_uid)
 
             if islands and row.n_count == 1:
                 shared = row.geometry.intersection(
@@ -157,12 +134,7 @@ def preprocess(
                 ).length
                 if shared == row.geometry.exterior.length:
                     uid = blg.iloc[row.neighbors[0]].mm_uid
-                    if uid in join:
-                        existing = join[uid]
-                        existing.append(row.mm_uid)
-                        join[uid] = existing
-                    else:
-                        join[uid] = [row.mm_uid]
+                    join.setdefault(uid, []).append(row.mm_uid)
 
         for key in tqdm(
             join, total=len(join), desc="Changing geometry", disable=not verbose
@@ -205,13 +177,10 @@ def remove_false_nodes(gdf):
     momepy.extend_lines
     momepy.close_gaps
     """
-    if isinstance(gdf, (gpd.GeoDataFrame, gpd.GeoSeries)):
+    if isinstance(gdf, gpd.GeoDataFrame | gpd.GeoSeries):
         # explode to avoid MultiLineStrings
         # reset index due to the bug in GeoPandas explode
-        if GPD_10:
-            df = gdf.reset_index(drop=True).explode(ignore_index=True)
-        else:
-            df = gdf.reset_index(drop=True).explode().reset_index(drop=True)
+        df = gdf.reset_index(drop=True).explode(ignore_index=True)
 
         # get underlying shapely geometry
         geom = df.geometry.array
@@ -240,7 +209,7 @@ def remove_false_nodes(gdf):
     unique, counts = np.unique(inp, return_counts=True)
     merge = res[np.isin(inp, unique[counts == 2])]
 
-    if len(merge) > 0:
+    if len(merge):
         # filter duplications and create a dictionary with indication of components to
         # be merged together
         dups = [item for item, count in collections.Counter(merge).items() if count > 1]
@@ -256,20 +225,16 @@ def remove_false_nodes(gdf):
             components[a[1]] = i
 
         # iterate through components and create new geometries
+        all_keys = {}
+        for k, v in components.items():
+            all_keys.setdefault(v, []).append(k)
         new = []
-        for c in set(components.values()):
-            keys = []
-            for item in components.items():
-                if item[1] == c:
-                    keys.append(item[0])
+        for keys in all_keys.values():
             new.append(shapely.line_merge(shapely.union_all(geom[keys])))
 
         # remove incorrect geometries and append fixed versions
         df = df.drop(merge)
-        if GPD_10:
-            final = gpd.GeoSeries(new).explode(ignore_index=True)
-        else:
-            final = gpd.GeoSeries(new).explode().reset_index(drop=True)
+        final = gpd.GeoSeries(new).explode(ignore_index=True)
         if isinstance(gdf, gpd.GeoDataFrame):
             return pd.concat(
                 [
@@ -496,10 +461,7 @@ def extend_lines(gdf, tolerance, target=None, barrier=None, extension=0):
     """
     # explode to avoid MultiLineStrings
     # reset index due to the bug in GeoPandas explode
-    if GPD_10:
-        df = gdf.reset_index(drop=True).explode(ignore_index=True)
-    else:
-        df = gdf.reset_index(drop=True).explode().reset_index(drop=True)
+    df = gdf.reset_index(drop=True).explode(ignore_index=True)
 
     if target is None:
         target = df
@@ -637,7 +599,7 @@ def _extend_line(coords, target, tolerance, snap=True):
             else:
                 new_point_coords = shapely.get_coordinates(intersection[0])
             coo = np.append(coords, new_point_coords)
-            new = np.reshape(coo, (int(len(coo) / 2), 2))
+            new = np.reshape(coo, (len(coo) // 2, 2))
 
             return new
         return coords
@@ -782,10 +744,7 @@ def _selecting_rabs_from_poly(
         rab["rab_diameter"] = rab[["deltax", "deltay"]].max(axis=1)
 
         # selecting the adjacent areas that are of smaller than itself
-        if GPD_10:
-            rab_adj = gpd.sjoin(gdf, rab, predicate="intersects")
-        else:
-            rab_adj = gpd.sjoin(gdf, rab, op="intersects")
+        rab_adj = gpd.sjoin(gdf, rab, predicate="intersects")
 
         area_right = area_col + "_right"
         area_left = area_col + "_left"
@@ -894,15 +853,14 @@ def _selecting_incoming_lines(rab_multipolygons, edges, angle_threshold=0):
     is used to select the line to be extended further.
     """
     # selecting the lines that are touching but not covered by
-    if GPD_10:
-        touching = gpd.sjoin(edges, rab_multipolygons, predicate="touches")
-        edges_idx, rabs_idx = rab_multipolygons.sindex.query_bulk(
+    touching = gpd.sjoin(edges, rab_multipolygons, predicate="touches")
+    if GPD_GE_013:
+        edges_idx, _ = rab_multipolygons.sindex.query(
             edges.geometry, predicate="covered_by"
         )
     else:
-        touching = gpd.sjoin(edges, rab_multipolygons, op="touches")
-        edges_idx, rabs_idx = rab_multipolygons.sindex.query_bulk(
-            edges.geometry, op="covered_by"
+        edges_idx, _ = rab_multipolygons.sindex.query_bulk(
+            edges.geometry, predicate="covered_by"
         )
     idx_drop = edges.index.take(edges_idx)
     touching_idx = touching.index
@@ -1059,12 +1017,6 @@ def roundabout_simplification(
         GeoDataFrame with an updated geometry and an additional
         column labeling modified edges.
     """
-    if not GPD_09:
-        raise ImportError(
-            "`roundabout_simplification` requires geopandas 0.9.0 or newer. "
-            f"Your current version is {gpd.__version__}."
-        )
-
     if len(edges[edges.geom_type != "LineString"]) > 0:
         raise TypeError(
             "Only LineString geometries are allowed. "
@@ -1087,3 +1039,190 @@ def roundabout_simplification(
     output = _ext_lines_to_center(edges, incoming_all, idx_drop)
 
     return output
+
+
+class FaceArtifacts:
+    """Identify face artifacts in street networks
+
+    For a given street network composed of transportation-oriented geometry containing
+    features representing things like roundabouts, dual carriegaways and complex
+    intersections, identify areas enclosed by geometry that is considered a `face
+    artifact` as per :cite:`fleischmann2023`. Face artifacts highlight areas with a high
+    likelihood of being of non-morphological (e.g. transporation) origin and may require
+    simplification prior morphological analysis. See :cite:`fleischmann2023` for more
+    details.
+
+    Parameters
+    ----------
+    gdf : geopandas.GeoDataFrame
+        GeoDataFrame containing street network represented as (Multi)LineString geometry
+    index : str, optional
+        A type of the shape compacntess index to be used. Available are
+        ['circlular_compactness', 'isoperimetric_quotient', 'diameter_ratio'], by
+        default "circular_compactness"
+    height_mins : float, optional
+        Required depth of valleys, by default np.NINF
+    height_maxs : float, optional
+        Required height of peaks, by default 0.008
+    prominence : float, optional
+        Required prominence of peaks, by default 0.00075
+
+    Attributes
+    ----------
+    threshold : float
+        Identified threshold between face polygons and face artifacts
+    face_artifacts : GeoDataFrame
+        A GeoDataFrame of geometries identified as face artifacts
+    polygons : GeoDataFrame
+        All polygons resulting from polygonization of the input gdf with the
+        face_artifact_index
+    kde : scipy.stats._kde.gaussian_kde
+        Representation of a kernel-density estimate using Gaussian kernels.
+    pdf : numpy.ndarray
+        Probability density function
+    peaks : numpy.ndarray
+        locations of peaks in pdf
+    valleys : numpy.ndarray
+        locations of valleys in pdf
+
+    Examples
+    --------
+    >>> fa = momepy.FaceArtifacts(street_network_prague)
+    >>> fa.threshold
+    6.9634555986177045
+    >>> fa.face_artifacts.head()
+                                                 geometry  face_artifact_index
+    6   POLYGON ((-744164.625 -1043922.362, -744167.39...             5.112844
+    9   POLYGON ((-744154.119 -1043804.734, -744152.07...             6.295660
+    10  POLYGON ((-744101.275 -1043738.053, -744103.80...             2.862871
+    12  POLYGON ((-744095.511 -1043623.478, -744095.35...             3.712403
+    17  POLYGON ((-744488.466 -1044533.317, -744489.33...             5.158554
+    """
+
+    def __init__(
+        self,
+        gdf,
+        index="circular_compactness",
+        height_mins=-np.inf,
+        height_maxs=0.008,
+        prominence=0.00075,
+    ):
+        try:
+            from esda import shape
+        except (ImportError, ModuleNotFoundError) as err:
+            raise ImportError(
+                "The `esda` package is required. You can install it using "
+                "`pip install esda` or `conda install esda -c conda-forge`."
+            ) from err
+
+        # Polygonize street network
+        polygons = gpd.GeoSeries(
+            shapely.polygonize(  # polygonize
+                [gdf.unary_union]
+            ).geoms,  # get parts of the collection from polygonize
+            crs=gdf.crs,
+        ).explode(
+            ignore_index=True
+        )  # shoudln't be needed but doesn't hurt to ensure
+
+        # Store geometries as a GeoDataFrame
+        self.polygons = gpd.GeoDataFrame(geometry=polygons)
+        if index == "circular_compactness":
+            self.polygons["face_artifact_index"] = np.log(
+                shape.minimum_bounding_circle_ratio(polygons) * polygons.area
+            )
+        elif index == "isoperimetric_quotient":
+            self.polygons["face_artifact_index"] = np.log(
+                shape.isoperimetric_quotient(polygons) * polygons.area
+            )
+        elif index == "diameter_ratio":
+            self.polygons["face_artifact_index"] = np.log(
+                shape.diameter_ratio(polygons) * polygons.area
+            )
+        else:
+            raise ValueError(
+                f"'{index}' is not supported. Use one of ['circlular_compactness', "
+                "'isoperimetric_quotient', 'diameter_ratio']"
+            )
+
+        # parameters for peak/valley finding
+        peak_parameters = {
+            "height_mins": height_mins,
+            "height_maxs": height_maxs,
+            "prominence": prominence,
+        }
+        mylinspace = np.linspace(
+            self.polygons["face_artifact_index"].min(),
+            self.polygons["face_artifact_index"].max(),
+            1000,
+        )
+
+        self.kde = gaussian_kde(
+            self.polygons["face_artifact_index"], bw_method="silverman"
+        )
+        self.pdf = self.kde.pdf(mylinspace)
+
+        # find peaks
+        self.peaks, self.d_peaks = find_peaks(
+            x=self.pdf,
+            height=peak_parameters["height_maxs"],
+            threshold=None,
+            distance=None,
+            prominence=peak_parameters["prominence"],
+            width=1,
+            plateau_size=None,
+        )
+
+        # find valleys
+        self.valleys, self.d_valleys = find_peaks(
+            x=-self.pdf + 1,
+            height=peak_parameters["height_mins"],
+            threshold=None,
+            distance=None,
+            prominence=peak_parameters["prominence"],
+            width=1,
+            plateau_size=None,
+        )
+
+        # check if we have at least 2 peaks
+        condition_2peaks = len(self.peaks) > 1
+
+        # check if we have at least 1 valley
+        condition_1valley = len(self.valleys) > 0
+
+        conditions = [condition_2peaks, condition_1valley]
+
+        # if both these conditions are true, we find the artifact index
+        if all(conditions):
+            # find list order of highest peak
+            highest_peak_listindex = np.argmax(self.d_peaks["peak_heights"])
+            # find index (in linspace) of highest peak
+            highest_peak_index = self.peaks[highest_peak_listindex]
+            # define all possible peak ranges fitting our definition
+            peak_bounds = list(zip(self.peaks[:-1], self.peaks[1:], strict=True))
+            peak_bounds_accepted = [b for b in peak_bounds if highest_peak_index in b]
+            # find all valleys that lie between two peaks
+            valleys_accepted = [
+                v_index
+                for v_index in self.valleys
+                if any(v_index in range(b[0], b[1]) for b in peak_bounds_accepted)
+            ]
+            # the value of the first of those valleys is our artifact index
+            # get the order of the valley
+            valley_index = valleys_accepted[0]
+
+            # derive threshold value for given option from index/linspace
+            self.threshold = float(mylinspace[valley_index])
+            self.face_artifacts = self.polygons[
+                self.polygons.face_artifact_index < self.threshold
+            ]
+        else:
+            warnings.warn(
+                "No threshold found. Either your dataset it too small or the "
+                "distribution of the face artifact index does not follow the "
+                "expected shape.",
+                UserWarning,
+                stacklevel=2,
+            )
+            self.threshold = None
+            self.face_artifacts = None
